package cs315.yourname.hwk2;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Matrix;
import android.util.AttributeSet;
import android.util.Log;
import android.view.DragEvent;
import android.view.MotionEvent;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

/**
 * A starter template for an Android scan conversion mini-painter. Includes the logic for doing the scan conversions
 * 
 * This class provides a dedicated thread for drawing, though the actual work is still in the UI thread,
 * because it is user driven.
 * 
 * @author Joel, adapted from code by Dave Akers
 * @version Fall 2013
 */
public class MiniPaintView extends SurfaceView implements SurfaceHolder.Callback
{
	private static final String TAG = "MiniPaintView";

	public static final int POINT_MODE = 0; //mode number (supposedly faster than enums)
	public static final int LINE_MODE = 1;
	public static final int CIRCLE_MODE = 2;
	public static final int POLYLINE_MODE = 3;
	public static final int RECTANGLE_MODE = 4;
	public static final int FLOOD_FILL_MODE = 5;
	public static final int AIRBRUSH_MODE = 6;

	private static final int PIXEL_SIZE = 2; //how "big" to make each pixel; change this for debugging
	private boolean DELAY_MODE = false; //whether to show a delay on the drawing (for debugging)
	private static final int DELAY_TIME = 5; //how long to pause between pixels; delay in ms; 5 is short, 50 is long

	private SurfaceHolder _holder; //basic drawing structure
	private DrawingThread _thread;
	private Context _context;
	
	private Bitmap _bmp; //frame buffer
	private int _width; //size of the image buffer
	private int _height;
	private Matrix _scaleM; //scale based on pixel size

	private int _mode; //drawing mode
	private int _color; //current painting color
	
	private int _startX; //starting points for multi-click operations
	private int _startY;


	/**
	 * Respond to touch events
	 */
	@Override
	public boolean onTouchEvent(MotionEvent event)
	{
		int x = (int)event.getX()/PIXEL_SIZE; //scale event to the size of the frame buffer!
		int y = (int)event.getY()/PIXEL_SIZE;

		switch(_mode) {
		case POINT_MODE:
			drawPoint(x,y);
			break;
		case LINE_MODE: 
			if(_startX < 0) { //see if we have a "first click" set of coords
				_startX = x; 
				_startY = y;
			}
			else {
				drawLine(_startX, _startY, x, y);
				_startX = -1;
			}
			break;
		case CIRCLE_MODE:
			if(_startX < 0) { //see if we have a "first click" set of coords
				_startX = x; 
				_startY = y;
			}
			else {
				int radius = (int) Math.sqrt((x - _startX)*(x - _startX) + (y - _startY)*(y - _startY));
				drawCircle(_startX, _startY, radius);
				_startX = -1;
			}
			break;
		case POLYLINE_MODE: 
			// TODO: POLYLINE_MODE not yet implemented!

			break;
		case RECTANGLE_MODE:
			// TODO: RECTANGLE_MODE not yet implemented!

			break;
		case FLOOD_FILL_MODE:
			// TODO: FLOOD_FILL_MODE not yet implemented!
			
			break;
		case AIRBRUSH_MODE:
			// TODO: AIRBRUSH_MODE not yet implemented!

			break;
		}

		return super.onTouchEvent(event); //pass up the tree, as needed
	}

	/**
	 * Draws a single point on the screen in the current paint color
	 * @param x x-coord of the point
	 * @param y y-coord of the point
	 */
	public void drawPoint(int x, int y)
	{
		setPixel(x, y); //I've done this one for you. You're welcome ;)
	}

	/**
	 * Draws a line on the screen in the current paint color
	 * @param startX x-coord of starting point
	 * @param startY y-coord of starting point
	 * @param endX x-coord of ending point
	 * @param endY y-coord of ending point
	 */
	public void drawLine(int startX, int startY, int endX, int endY)
	{
		// TODO: Implement this method (Bresenham line algorithm)
	
	}

	/**
	 * Draws a circle on the screen in the current paint color
	 * @param x x-coord of circle center
	 * @param y y-coord of circle center
	 * @param radius radius of the circle
	 */
	public void drawCircle(int x, int y, int radius)
	{
		// TODO: Implement this method (Bresenham circle algorithm)

	}

	/**
	 * Draws a rectangle on the screen in the current paint color
	 * @param startX x-coord of first corner (i.e., upper left)
	 * @param startY y-coord of first corner
	 * @param endX x-coord of second corner (i.e., lower right)
	 * @param endY y-coord of second corner
	 */
	public void drawRectangle(int startX, int startY, int endX, int endY)
	{
		// TODO: Implement this method! 
		// Note that you can reuse your drawLine() method, but you shouldn't reuse the floodFill() method!
		
	}

	/**
	 * Flood-fills a space on the canvas in the current paint color
	 * @param x x-coord to start filling from
	 * @param y y-coord to start filling from
	 */
	public void floodFill(int x, int y)
	{
		// TODO: Implement this method!
		// Note that you should implement this iteratively (using a Stack), not recursively!
		
	}

	/**
	 * Draws an airbrushed blob in the current paint color (blending with existing colors)
	 * @param x x-coord to center the airbrush
	 * @param y y-coord to center the airbrush
	 */
	public void airBrush(int x, int y)
	{
		// TODO: Implement this method!
	}


	/*********
	 * You shouldn't need to modify anything below this point!
	 ********/
	
	/**
	 * We need to override all the constructors, since we don't know which will be called
	 * All the constructors eventually call init()
	 */
	public MiniPaintView(Context context) {
		this(context, null);
	}

	public MiniPaintView(Context context, AttributeSet attrs) {
		this(context, attrs, 0);
	}

	public MiniPaintView(Context context, AttributeSet attrs, int defaultStyle) {
		super(context, attrs, defaultStyle);
		init(context);
	}

	/**
	 * Our initialization method (called from constructors)
	 */
	public void init(Context context)
	{
		_context = context;
		_holder = this.getHolder(); //handles control of the surface
		_holder.addCallback(this);
		_thread = new DrawingThread(_holder, this);

		_scaleM = new Matrix();
		_scaleM.setScale(PIXEL_SIZE, PIXEL_SIZE);
		DELAY_MODE = true;
		
		_mode = POINT_MODE;
		_color = Color.WHITE;

		_startX = -1; //initialize as invalid
		_startY = -1;

		setFocusable(true); //just in case for touch events
	}
	
	
	/**
	 * Sets the drawing mode (UI method)
	 */
	public void setMode(int mode)
	{
		_mode = mode;
		_startX = -1;
		_startY = -1;
		//Toast toast = Toast.makeText(_context, "Mode set: "+_mode, Toast.LENGTH_SHORT);
		//toast.show();
	}

	/**
	 * Sets the painting color (UI method)
	 */
	public void setColor(int color)
	{
		_color = color;
		//Toast toast = Toast.makeText(_context, "Color set: "+_color, Toast.LENGTH_SHORT);
		//toast.show();
	}

	/**
	 * Clears the drawing (resets all pixels to Black)
	 */
	public void clearDrawing()
	{
		for(int i=0; i<_width; i++)
			for(int j=0; j<_height; j++)
				_bmp.setPixel(i, j, Color.BLACK);
	}

	/**
	 * Helper method to set a single pixel to a given color.
	 * Performed clipping, and includes debug settings to introduce a delay in pixel drawing
	 * @param x x-coord of pixel
	 * @param y y-coord of pixel
	 * @param color color to apply to pixel
	 */
	public void setPixel(int x, int y, int color)
	{
		/*Can comment out this block to make things go even faster*/
		if(DELAY_MODE) //if we're in delay mode, then pause while drawing
		{
			try{
				Thread.sleep(DELAY_TIME);
			} catch (InterruptedException e){}
		}
		
		if(x >= 0 && x < _width && y >= 0 && y < _height) //clipping for generated shapes (so we don't try and draw outside the bmp)
			_bmp.setPixel(x, y, color);
	}

	/**
	 * Helper method to set a single pixel to the current paint color.
	 * Performed clipping, and includes debug settings to introduce a delay in pixel drawing
	 * @param x x-coord of pixel
	 * @param y y-coord of pixel
	 */
	public void setPixel(int x, int y)
	{
		setPixel(x,y,_color);
	}
	
	/**
	 * Convenience method to get the color of a specific pixel
	 * @param x x-coord of pixel
	 * @param y y-coord of pixel
	 * @return The color of the pixel
	 */
	public int getPixel(int x, int y)
	{
		return _bmp.getPixel(x,y);
	}
	
	//called when the surface changes (like sizes changes due to rotate). Will need to respond accordingly.
	@Override
	public void surfaceChanged(SurfaceHolder holder, int format, int width, int height)
	{
		//store new size for our BitMap
		_width = width/2;
		_height = height/2;

		//create a properly-sized bitmap to draw on
		_bmp = Bitmap.createBitmap(_width, _height, Bitmap.Config.ARGB_8888);
	}

	@Override
	public void surfaceCreated(SurfaceHolder holder) { //initialization stuff
		_thread.setRunning(true);
		_thread.start();
	}

	@Override
	public void surfaceDestroyed(SurfaceHolder holder) { //cleanup
		//Tell the thread to shut down, but wait for it to stop!
		_thread.setRunning(false);
		boolean retry = true;
		while(retry) {
			try {
				_thread.join();
				retry = false;
			} catch (InterruptedException e) {
				//will try again...
			}
		}
		Log.d(TAG, "Drawing thread shut down.");
	}

	/**
	 * An inner class representing a thread that does the drawing. Animation timing could go in here.
	 * http://obviam.net/index.php/the-android-game-loop/ has some nice details about using timers to specify animation
	 */
	public class DrawingThread extends Thread 
	{
		private boolean _isRunning; //whether we're running or not (so we can "stop" the thread)
		private SurfaceHolder _holder; //the holder we're going to post updates to
		private MiniPaintView _view; //the view that has drawing details

		/**
		 * Constructor for the Drawing Thread
		 * @param holder
		 * @param view
		 */
		public DrawingThread(SurfaceHolder holder, MiniPaintView view)
		{
			super();
			this._holder = holder;
			this._view = view;
			this._isRunning = false;
		}

		/**
		 * Executed when we call thread.start()
		 */
		@Override
		public void run()
		{
			Canvas canvas;
			while(_isRunning)
			{
				canvas = null;
				try {
					canvas = _holder.lockCanvas();
					synchronized (_holder) {
						canvas.drawBitmap(_bmp,_scaleM,null); //draw the _bitmap onto the canvas. Note that filling the bitmap occurs elsewhere
					}
				} finally { //no matter what (even if something goes wrong), make sure to push the drawing so isn't inconsistent
					if (canvas != null) {
						_holder.unlockCanvasAndPost(canvas);
					}
				}
			}
		}

		/**
		 * Public toggle for whether the thread is running.
		 * @param isRunning
		 */
		public void setRunning(boolean isRunning){
			this._isRunning = isRunning;
		}
	}
}
